  function [B, XN, Pr, wo] = coarsestep2(D, X, wx)
% function [B, XN, Pr, wo] = coarsestep2(D, X, wx)   
% Algorithm proceeds by traversing EDGES - based on D
%%    (largest to smallest weights in D)     
% IN
%  X   = data matrix - each columns contains a point (e.g, xyz
%        coordinates or some other features);
%  D   = weight matrix..  
%  wx  = weights of vertices [row vector]
% OUT: 
%  B    = Output sparse  matrix of weights
%  XN   = coarse nodes (coarsened `features')
%  prnt = parents array : node i in old graph has parent prnt(i) 
%        in coarse graph 
% wo    = weights for output nodes  [row vector]
%% ASSUMES DIAGONAL OF X IS ZERO.

N = size(D,1);     % Numbers of nodes of the given (fine) graph.
prnt = zeros(N,1); % parents list 
%%-------------------- unroll sparse matrix into arrays.
[I, J, W] = find(D);
nz = length(J);

[~, indx] = sort(W,'descend');

%[dummy, indx] = sort(W,'descend');
%%indx(1:200)
%%%xindx = randperm(nz) ;
new = 0;
%%indx = randperm(nz);
kp = 0;
for k = 1:nz 
  next = indx(k);
  i = I(next);
  if (prnt(i) >0), continue, end;
  j = J(next);
%%-------------------- exclude i->i links
  if (i <= j), continue, end;
  if (prnt(j) >0), continue, end;
%%-------------------- create new node 
  new = new+1;
%%disp(new)
%%-------------------- parents of this new node 
  prnt(i) = new;
  prnt(j) = new;
%%-------------------- projection matrix
%%  1st
  kp = kp+1;
  ip(kp) = i;
  jp(kp) = new;
%% 2nd
  kp = kp+1;
  ip(kp) = j;
  jp(kp) = new;
end
%%-------------------- mop up - left-over singletons -- ADD TO MOST 
for ii=1:N
    if (prnt(ii) ~= 0), continue, end 
%%-------------------- real singleton -- zero row  
    if (nnz(D(:,ii)) ==0) 
      new = new+1;
      prnt(ii) = new;
%%-------------------- update projection matrix:
      kp = kp+1;
      ip(kp) = ii;
      jp(kp) = new;
    else
%%-------------------- find best candidate among neigbors to add as parent
      [~, idx] = max(D(:,ii));
      prnt(ii) = prnt(idx);
    end
end
%%-------------------- now build the projection graph;
Pr = sparse(ip, jp, 1, N, new);
for j=1:new
  [ir, ~, ~] = find(Pr(:,j));
  t = norm(wx(ir));
  Pr(ir,j)  = wx(ir)'/t;
end
%%-------------------- reorder Pr to best preserve original structure 
for j=1:new
  ir(j) = min(find(Pr(:,j)));
end
[~, idx] = sort(ir);
Pr = Pr(:,idx);
%%-------------------- return modified objects
XN = X*Pr;
wo = wx*Pr;
B = Pr'*D*Pr;
